/**
 * 
 * Last changed: $Date$ By: $Author$ URL: $HeadURL$ $Revision$
 *
 */
package de.desy.mst.libs.framework.mstapp.appinf;

import java.lang.reflect.Constructor;

import de.desy.mst.libs.framework.mstapp.appinf.client.ClientFactory;
import de.desy.mst.libs.framework.mstapp.appinf.ide.DebugFactory;
import de.desy.mst.libs.framework.mstapp.appinf.jaws.JawsFactory;
import de.desy.mst.libs.framework.mstapp.appinf.server.ServerFactory;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * @author labudda
 *
 */
public enum ApplikationKind {
  /*
   * Die Reihenfolge der Arten ist fuer den Fall dass die Art nicht explizit definiert ist relevant.
   * Es wird vom letzten zum ersten ueberprueft, ob eine entsprechende Applikation erkannt wurde
   * Jaws und Server unterscheiden sich nur darin, dass Jaws entsprechende System Properties definiert 
   */
  Konsole(new ClientFactory()), Server(new ServerFactory()), Jaws(new JawsFactory()), Ide(
      new DebugFactory()), Automatic;

  private final static java.util.logging.Logger log =
      java.util.logging.Logger.getLogger(ApplikationKind.class.getName());

  private static class NoFactory extends AApplicationInterfaceFactory {



    /*
     * (non-Javadoc)
     * 
     * @see de.desy.mst.libs.IApplikationInterfaceFactory#getApplicationInformation()
     */
    @Override
    public IApp getApplicationInformation() throws IllegalStateException {
      throw new IllegalStateException("There's no factory defined");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.desy.mst.libs.framework.mstapp.appinf.AApplicationInterfaceFactory#
     * areKindSpecificRequirementsFulfilled()
     */
    @Override
    protected boolean areKindSpecificRequirementsFulfilled() {
      return false;
    }

  }


  /**
   * Welche factory erzeugt die Applikationsinformationen
   */
  private final IApplikationInterfaceFactory factory;

  /**
   * Default factory, nur relevant fuer Automatic Kind. Insbesondere fuer Debug ist es wichtig die
   * automatsche Auswahl zu beeinflussen. Dafuer kann die Property 'ApplicationKindFactory'
   * definiert werden. Der Wert der Property entspricht dem Klassennamen der factory
   */
  private ApplikationKind() {
    /*
     * Das Klassenmitglied "log" kann im Konstruktor nicht genutzt werden.
     * Cannot refer to the static enum field ApplikationKind.log within an initializer
     */
    final java.util.logging.Logger log =
        java.util.logging.Logger.getLogger(ApplikationKind.class.getName());
    if (CustomSystemProperty.ApplicationKindFactory.isAvailable()) {
      IApplikationInterfaceFactory definedApplicationFactory = null;
      try {
        final String akf = CustomSystemProperty.ApplicationKindFactory.getValue();
        log.finer("Using defined ApplicationKindFactory " + akf);
        final Class<?> definedClass = Class.forName(akf);
        final Class<?>[] types = null;
        final Object[] arguments = null;
        final Constructor<?> constructor = definedClass.getConstructor(types);
        definedApplicationFactory =
            (IApplikationInterfaceFactory) constructor.newInstance(arguments);
      } catch (final Exception e) {
        /*
         * Weil diese Option i.a. nur fuer debug releavant ist, darf diese Exception keinen
         * negativen Einfluss auf den Kontrollraum haben
         */
        log.log(java.util.logging.Level.WARNING, "Class defined by 'ApplicationKindFactory' not found :"
            + CustomSystemProperty.ApplicationKindFactory.getValue(), e);
      }
      this.factory = definedApplicationFactory;
    } else {
      this.factory = null;
    }
  }

  /**
   * An application kind with information factory
   * 
   * @param factory
   */
  private ApplikationKind(final IApplikationInterfaceFactory factory) {
    this.factory = factory;
  }

  /**
   * @return factory die "ihre" Umgebung erkennt
   */
  private IApplikationInterfaceFactory getFactory() {
    ApplikationKind.log.entering(ApplikationKind.class.getName(), "getFactory");
    IApplikationInterfaceFactory validFactory = this.factory; // nur bei Automatic ist die factory
                                                              // nicht definiert
    int checkFactoryIndex = ApplikationKind.values().length - 1;
    while (checkFactoryIndex >= 0 && validFactory == null) {
      final ApplikationKind checkKind = ApplikationKind.values()[checkFactoryIndex];
      if (checkKind.factory == null) {
        // ignore
        checkFactoryIndex--;
      } else if (checkKind.factory.isValidKind()) {
        validFactory = checkKind.factory;
        checkFactoryIndex = -1;
      } else {
        checkFactoryIndex--;
      }
    }
    if (validFactory == null) {
      ApplikationKind.log.warning("No application kind defined");
      return new NoFactory();
    } else {
      ApplikationKind.log.info(validFactory.getClass().getName() + " defined as Information factory");
      return validFactory;
    }
  }

  /**
   * @return the factory
   * @throws IllegalStateException
   * @see de.desy.mst.libs.framework.mstapp.appinf.IApplikationInterfaceFactory#getApplicationInformation()
   */
  public IApp getApplicationInformation() throws IllegalStateException {
    Logger.SWFW.debug("Enter: IApp ApplikationKind:getApplicationInformation()");
    return this.getFactory().getApplicationInformation();
  }

  /**
   * @return true on valid for this kind of applikation
   * @see de.desy.mst.libs.framework.mstapp.appinf.IApplikationInterfaceFactory#isValidKind()
   */
  public boolean isValidKind() {
    return this.getFactory().isValidKind();
  }

}
